@@ -12,6 +12,8 @@ from .base import *
 from matplotlib.gridspec import GridSpec
 from matplotlib.lines import Line2D
 
+from ...spikesorting import color_utils
+from ...spikesorting.sorting.tools import apply_descending_sort_with_waveform
 
 
 class ModelSpikeList(QAbstractItemModel):
@@ -164,19 +166,26 @@ class SpikeList(SpikeSortingWidgetBase):
         flags = QItemSelectionModel.Select #| QItemSelectionModel.Rows
         itemsSelection = QItemSelection()
         ind, = np.where(self.spikesorter.selected_spikes)
-        if ind.size>10000:
-            # only the first one because QT4 is able to handle with big selections
-            if ind.size>0:
-                for j in range(2):
-                    index = self.treeSpike.model().index(ind[0],j,QModelIndex())
-                    ir = QItemSelectionRange( index )
-                    itemsSelection.append( ir )
-        else:
-            for i in ind:
-                for j in range(2):
-                    index = self.treeSpike.model().index(i,j,QModelIndex())
-                    ir = QItemSelectionRange( index )
-                    itemsSelection.append( ir )
+        #~ if ind.size>1000:
+            #~ # only the first one because QT4 is able to handle with big selections
+            #~ if ind.size>0:
+                #~ for j in range(2):
+                    #~ index = self.treeSpike.model().index(ind[0],j,QModelIndex())
+                    #~ ir = QItemSelectionRange( index )
+                    #~ itemsSelection.append( ir )
+        #~ else:
+            #~ for i in ind:
+                #~ for j in range(2):
+                    #~ index = self.treeSpike.model().index(i,j,QModelIndex())
+                    #~ ir = QItemSelectionRange( index )
+                    #~ itemsSelection.append( ir )
+        if ind.size>100:
+            ind = ind[:10]
+        for i in ind:
+            for j in range(2):
+                index = self.treeSpike.model().index(i,j,QModelIndex())
+                ir = QItemSelectionRange( index )
+                itemsSelection.append( ir )
         self.treeSpike.selectionModel().select( itemsSelection , flags)
 
         # set selection visible
@@ -310,28 +319,46 @@ class UnitList(SpikeSortingWidgetBase):
         self.clusters_activation_changed.emit()
 
     def contextMenuNeuron(self, point):
-        menu = QMenu()
-        act = menu.addAction(QIcon(':/window-close.png'),u'Delete selection forever')
-        act.triggered.connect(self.deleteSelection)
-        act = menu.addAction(QIcon(':/user-trash.png'), u'Move selection to trash')
-        act.triggered.connect(self.moveToTrash)
-        act = menu.addAction(QIcon(':/merge.png'), u'Group selection in one unit')
-        act.triggered.connect(self.groupSelection)
-        act = menu.addAction(QIcon(':/color-picker.png'), u'Select these spikes')
-        act.triggered.connect(self.selectSpikeFromCluster)
-        act = menu.addAction(QIcon(':/go-jump.png'), u'Regroup small units')
-        act.triggered.connect(self.regroupSmallUnits)
-        act = menu.addAction(QIcon(':/TODO.png'), u'Hide/Show on ndviewer and waveform')
-        act.triggered.connect(self.hideOrShowClusters)
+        n = len(self.tableNeuron.selectedIndexes())/self.tableNeuron.columnCount ()
+        self.menu = menu = QMenu()
+        #~ print 'n', n
+        if n>=0: 
+            act = menu.addAction(QIcon(':/applications-graphics.png'), u'Recolorize all cluster')
+            act.triggered.connect(self.recolorizeCluster)
+            act = menu.addAction(QIcon(':/view-filter.png'), u'Sort by ascending waveform power')
+            act.triggered.connect(self.sortCluster)
+            act = menu.addAction(QIcon(':/TODO.png'), u'Show all')
+            act.triggered.connect(self.showAll)
+            act = menu.addAction(QIcon(':/TODO.png'), u'Hide all')
+            act.triggered.connect(self.hideAll)
+            
+        if n>=1:
+            act = menu.addAction(QIcon(':/window-close.png'),u'Delete selection forever')
+            act.triggered.connect(self.deleteSelection)
+            act = menu.addAction(QIcon(':/user-trash.png'), u'Move selection to trash')
+            act.triggered.connect(self.moveToTrash)
+            act = menu.addAction(QIcon(':/merge.png'), u'Group selection in one unit')
+            act.triggered.connect(self.groupSelection)
+            act = menu.addAction(QIcon(':/color-picker.png'), u'Select these spikes')
+            act.triggered.connect(self.selectSpikeFromCluster)
+            act = menu.addAction(QIcon(':/go-jump.png'), u'Regroup small units')
+            act.triggered.connect(self.regroupSmallUnits)
         
-        if len(self.tableNeuron.selectedIndexes()) ==  self.tableNeuron.columnCount():
+        #~ act = menu.addAction(QIcon(':/TODO.png'), u'Hide/Show on ndviewer and waveform')
+        #~ act.triggered.connect(self.hideOrShowClusters)
+        
+        if n==1:
             # one selected row only
             #~ act = menu.addAction(QIcon(':/Clustering.png'), u'Explode cluster (sub clustering)')
             #~ act.triggered.connect(self.subComputeCluster)
-            act = menu.addAction(QIcon(':/TODO.png'), u'Set name of this unit')
-            act.triggered.connect(self.setUnitNameAndScore)
+            act = menu.addAction(QIcon(':/TODO.png'), u'Set name/color/score of this unit')
+            act.triggered.connect(self.setUnitNameColorScore)
+        
+        #~ if menu.exec_(self.cursor().pos()):
+        menu.popup(self.cursor().pos())
+            #~ pass
+            #~ print 'yep'
         
-        menu.exec_(self.cursor().pos())
     
     def deleteSelection(self):
         for index in self.tableNeuron.selectedIndexes():
@@ -376,6 +403,7 @@ class UnitList(SpikeSortingWidgetBase):
             #~ print sps.spike_clusters.shape
             sps.selected_spikes[sps.spike_clusters == self.cluster_list[index.row()]] = True
         self.spike_selection_changed.emit()
+        #~ print 'yep'
 
     #TODO
     #~ def subComputeCluster(self):
@@ -420,19 +448,19 @@ class UnitList(SpikeSortingWidgetBase):
         self.refresh()
         self.spike_clusters_changed.emit()
     
-    def hideOrShowClusters(self):
-        sps = self.spikesorter
-        for index in self.tableNeuron.selectedIndexes():
-            if index.column() !=0: continue
-            r = index.row()
-            c = self.cluster_list[r]
-            if c in sps.cluster_displayed_subset and sps.cluster_displayed_subset[c].size == 0:
-                sps.random_display_subset(c)
-            else:
-                sps.cluster_displayed_subset[c]  = np.array([ ], 'i')
-        self.spike_subset_changed.emit()
+    #~ def hideOrShowClusters(self):
+        #~ sps = self.spikesorter
+        #~ for index in self.tableNeuron.selectedIndexes():
+            #~ if index.column() !=0: continue
+            #~ r = index.row()
+            #~ c = self.cluster_list[r]
+            #~ if c in sps.cluster_displayed_subset and sps.cluster_displayed_subset[c].size == 0:
+                #~ sps.random_display_subset(c)
+            #~ else:
+                #~ sps.cluster_displayed_subset[c]  = np.array([ ], 'i')
+        #~ self.spike_subset_changed.emit()
     
-    def setUnitNameAndScore(self):
+    def setUnitNameColorScore(self):
         sps = self.spikesorter
         for index in self.tableNeuron.selectedIndexes():
             if index.column() !=0: continue
@@ -447,10 +475,41 @@ class UnitList(SpikeSortingWidgetBase):
         class Parameters(DataSet):
             name = StringItem('Name of unit {}'.format(c), default ='')
             #~ sorting_score = FloatItem('Score of unit {}'.format(c), default =sorting_score)
+            color = ColorItem('Color of unit {}'.format(c), default= color_utils.mpl_to_html( sps.cluster_colors[c] ))
         d =  ParamDialog(Parameters, title = 'Name')
         d.update(dict(name = name))
         if d.exec_():
             sps.cluster_names[c] = d.to_dict()['name']
             #~ sps.sortingScores[c] = d.get_dict()['sortingScore']
+            color = d.to_dict()['color']
+            sps.cluster_colors[c] = color_utils.html_to_mplRGB(color)
+            self.refresh()
+            self.clusters_color_changed.emit()
+            
+    
+    def recolorizeCluster(self):
+        self.spikesorter.refresh_colors(reset = True)
         self.refresh()
+        self.clusters_color_changed.emit()
+    
+    def sortCluster(self):
+        apply_descending_sort_with_waveform(self.spikesorter)
+        self.spikesorter.refresh_colors(reset = True)
+        self.refresh()
+        self.spike_clusters_changed.emit()
+    
+    def showHideAll(self, state):
+        sps = self.spikesorter
+        for c in sps.cluster_names:
+            sps.active_cluster[c] = state
+        self.refresh()
+        self.clusters_activation_changed.emit()
+        
+    
+    def showAll(self):
+        self.showHideAll(True)
+    
+    def hideAll(self):
+        self.showHideAll(False)
+
 
