@@ -6,13 +6,15 @@ Theses widget display individual spike on signals.
 
 
 
-
+from matplotlib.gridspec import GridSpec
 
 from .base import *
 
 from ..viewers import TimeSeeker
 from ..guiutil import *
 
+
+
 import pyqtgraph as pg
 
 class MyViewBox(pg.ViewBox):
@@ -65,6 +67,7 @@ class SignalAndSpike(SpikeSortingWidgetBase):
         
         self.plots = [ ]
         self.curves = [ ]
+        self.threshold_curves =[ ]
         for i in range(sps.trodness):
             l.nextRow()
             viewBox = MyViewBox()
@@ -79,6 +82,12 @@ class SignalAndSpike(SpikeSortingWidgetBase):
             viewBox.yzoom_in.connect(lambda : self.yzoom(.8))
             viewBox.yzoom_out.connect(lambda : self.yzoom(1.2))
             viewBox.clicked_at_x.connect(self.clicked_at_x)
+            
+            tc = pg.InfiniteLine(angle = 0, movable = False, pen = 'g')
+            tc.setPos(0.)
+            self.threshold_curves.append(tc)
+            plot.addItem(tc)
+            tc.hide()
         
         self.scatters = [ {} for i in range(sps.trodness) ]
         
@@ -92,7 +101,8 @@ class SignalAndSpike(SpikeSortingWidgetBase):
 
         param_global = [
                         {'name': 'auto_zoom_on_select', 'type': 'bool', 'value': True },
-                        {'name': 'zoom_size', 'type': 'float', 'value':  0.05, 'step' : 0.001 },
+                        {'name': 'zoom_size', 'type': 'float', 'value':  0.2, 'step' : 0.001 },
+                        {'name': 'plot_threshold', 'type': 'bool', 'value':  True },
                         ]        
         self.params = pg.parametertree.Parameter.create( name='Global options', type='group',
                                                     children = param_global)
@@ -172,6 +182,8 @@ class SignalAndSpike(SpikeSortingWidgetBase):
     
 
     def seek(self, t):
+        #~ self.act_enable_spike_selection.setChecked(False)
+        
         sps = self.spikesorter
         s =  self.combo.currentIndex()
         if sps.spike_index_array is not None:
@@ -189,11 +201,13 @@ class SignalAndSpike(SpikeSortingWidgetBase):
         ind_stop = int(np.rint((t2-sps.seg_t_start[s].rescale('s').magnitude)*sr))
         if ind_start<0:
             ind_start=0
-            t1 = float(sps.seg_t_start[s].magnitude)
+            _t1 = float(sps.seg_t_start[s].magnitude)
+        else:
+            _t1 = t1
         if ind_stop>self.sigs[0,s].size:
             ind_stop=self.sigs[0,s].size
-            t2 = float(sps.seg_t_stop[s].magnitude)
-        t_vect = np.arange(0,ind_stop-ind_start, dtype='f')/sr+t1
+            #~ t2 = float(sps.seg_t_stop[s].magnitude)
+        t_vect = np.arange(0,ind_stop-ind_start, dtype='f')/sr+_t1
         for i in range(sps.trodness):
             if t_vect.size:
                 self.curves[i].setData(t_vect, self.sigs[i,s][ind_start:ind_stop])
@@ -216,7 +230,15 @@ class SignalAndSpike(SpikeSortingWidgetBase):
                     self.plots[i].addItem(scatter['sel'])
                 else :
                     scatter['sel'].setData(t_vect[ind-ind_start], self.sigs[i,s][ind])
-
+            
+            # remove old ones
+            for i, scatter in enumerate(self.scatters):
+                for c in scatter.keys():
+                    if c=='sel' : continue
+                    if c not in sps.cluster_names:
+                        self.plots[i].removeItem(scatter[c])
+                        scatter.pop(c)
+            
             # Spikes by cluster
             vpos = pos[inwindow]
             for c in sps.cluster_names.keys():
@@ -237,7 +259,19 @@ class SignalAndSpike(SpikeSortingWidgetBase):
                             self.plots[i].addItem(scatter[c])
                             scatter[c].sigClicked.connect(self.item_clicked)
                         #~ scatter[c].vb = self.plots[i].vb
-
+        
+        for i in range(sps.trodness):
+            tc = self.threshold_curves[i]
+            if isinstance(self, FilteredBandSignal) and hasattr(sps, 'detection_thresholds') and\
+                                sps.detection_thresholds is not None and\
+                                self.params['plot_threshold']:
+                tc.setPos(sps.detection_thresholds[i,s])
+                tc.show()
+            else:
+                tc.setPos(0.)
+                tc.hide()
+                
+        
         for plot in self.plots:
             #~ print t1, t2
             plot.setXRange( t1, t2, padding = 0.0)
@@ -374,8 +408,101 @@ class FilteredBandSignal(SignalAndSpike):
 
 
 
+class BetweenSpikeNoiseEstimation(SpikeSortingWidgetBase):
+    name = 'Between spike snoise estimation'
+    refresh_on = [ 'spike_index_array', ]
+    icon_name = 'plot-waveform.png'
+    
+    plot_dataset = type('Parameters', (DataSet,), { 'deviation_coeff' : FloatItem('deviation coeff',  default = 2.5), 
+                                                                                                    'maxiter' : IntItem('maxiter for random chunk',  default = 5), 
+                                                                                                    'nb_chunk' : IntItem('nb_chunk',  default = 1000), 
+                                                                                                })
 
 
+    def __init__(self,**kargs):
+        super(BetweenSpikeNoiseEstimation, self).__init__(**kargs)
+        self.canvas = SimpleCanvasAndTool( )
+        self.canvas.toolbar.hide()
+        self.canvas.toolbar.pan()
+        #~ self.canvas = SimpleCanvas( )
+        self.mainLayout.addWidget(self.canvas)
+        self.fig = self.canvas.fig
+
+        sps = self.spikesorter
+
+        # create axes
+        self.axs = [ ]
+        self.ax2s = [ ]
+        ax = None
+        ax2 = None
+        grid = GridSpec(3,sps.trodness, wspace=0, hspace=0)
+        for i in range(sps.trodness):
+            ax = self.fig.add_subplot( grid[0:2,i] ,  sharex = ax, sharey = ax)
+            self.axs.append( ax )
+            #~ ax.axvline(0, color = 'r', ls = '--', alpha = .7)
+            ax2 = self.fig.add_subplot( grid[2,i] ,  sharex = ax, sharey = ax2)
+            self.ax2s.append(ax2)
+            #~ ax2.axvline(0, color = 'r', ls = '--', alpha = .7)
+            ax.get_xaxis().set_visible(False)
+            if i !=0:
+                ax.get_yaxis().set_visible(False)
+                ax2.get_yaxis().set_visible(False)
+            ax2.set_xticks(np.arange(-10,10))
+            ax2.set_xticklabels(['']*20)
+            
+            self.plot_options_changed.connect(self.on_plot_options_changed)
+    
+    def on_plot_options_changed(self):
+        sps = self.spikesorter
+        sps.interspike_noise_median = None
+        self.refresh()
+    
+    def refresh(self):
+        sps = self.spikesorter
+        for i in range(sps.trodness):
+            self.axs[i].clear()
+            self.ax2s[i].clear()
+            #~ self.axs[i].set_xticks(np.arange(-10,10))
+            #~ self.axs[i].set_xticklabels(['']*20)
+            self.axs[i].set_xticks([])
+            if i==0:
+                self.axs[i].set_ylabel('Median')
+                self.ax2s[i].set_ylabel('MAD')
+            
+            
+        
+        if sps.spike_index_array is None : 
+            self.canvas.draw()
+            return
+        
+        if sps.interspike_noise_median is None:
+            maxiter = self.plot_parameters['maxiter']
+            nb_chunk = self.plot_parameters['nb_chunk']
+            
+            sps.recompute_interspike_noise(n = nb_chunk, maxiter = maxiter)
+        
+        #~ times = (np.arange(-sps.left_sweep, sps.right_sweep+1)/sps.wf_sampling_rate).rescale('ms').magnitude
+        
+        coeff = self.plot_parameters['deviation_coeff']
+        
+        for i in range(sps.trodness):
+            m =  sps.interspike_noise_median[i,:]
+            sd = sps.interspike_noise_mad[i,:]
+            times = np.arange(m.size)
+            self.axs[i].plot(times, m, linewidth=2,color = 'k',)
+            self.ax2s[i].plot(times, sd, linewidth=2,color = 'k',)
+            self.axs[i].fill_between(times, m-sd*coeff, m+sd*coeff ,
+                                    color = 'k', alpha = .3)
+            
+            self.axs[i].axvline(0, color = 'r', ls = '--', alpha = .7)
+            self.ax2s[i].axvline(0, color = 'r', ls = '--', alpha = .7)
+                
+        self.axs[0].set_xlim(times[0], times[-1])
+
+        self.canvas.draw()
+        
+        
+
 
 
 class SignalStatistics(SpikeSortingWidgetBase):
