@@ -17,61 +17,119 @@ class AverageWaveforms(SpikeSortingWidgetBase):
     refresh_on = [  'spike_waveforms', 'spike_clusters', 'cluster_names']
     icon_name = 'plot-waveform.png'
     
+    plot_dataset = type('Parameters', (DataSet,), { 'estimation' : ChoiceItem('max_waveform_by_cluster',  ['Median + MAD', 'Mean + STD']),
+                                                                                                'deviation_coeff' : FloatItem('deviation coeff',  default = 2.5),
+                                                                                                })
     
     def __init__(self,**kargs):
         super(AverageWaveforms, self).__init__(**kargs)
         self.canvas = SimpleCanvasAndTool( )
+        self.canvas.toolbar.hide()
+        self.canvas.toolbar.pan()
         #~ self.canvas = SimpleCanvas( )
         self.mainLayout.addWidget(self.canvas)
         self.fig = self.canvas.fig
-        self.refresh()
-
-    def refresh(self):
+        #~ self.refresh()
+        
         sps = self.spikesorter
-        self.fig.clear()
-        if sps.spike_waveforms is None : return
-        
-        # recreate axes
         
+        # create axes
         self.axs = [ ]
         self.ax2s = [ ]
         ax = None
         ax2 = None
-        grid = GridSpec(4,sps.trodness)
+        grid = GridSpec(3,sps.trodness, wspace=0, hspace=0)
         for i in range(sps.trodness):
-            ax = self.fig.add_subplot( grid[0:3,i] ,  sharex = ax, sharey = ax)
+            ax = self.fig.add_subplot( grid[0:2,i] ,  sharex = ax, sharey = ax)
             self.axs.append( ax )
-            ax.axvline(0, color = 'r', ls = '--', alpha = .7)
-            ax2 = self.fig.add_subplot( grid[3,i] ,  sharex = ax, sharey = ax2)
+            #~ ax.axvline(0, color = 'r', ls = '--', alpha = .7)
+            ax2 = self.fig.add_subplot( grid[2,i] ,  sharex = ax, sharey = ax2)
             self.ax2s.append(ax2)
-            ax2.axvline(0, color = 'r', ls = '--', alpha = .7)
+            #~ ax2.axvline(0, color = 'r', ls = '--', alpha = .7)
             ax.get_xaxis().set_visible(False)
             if i !=0:
                 ax.get_yaxis().set_visible(False)
                 ax2.get_yaxis().set_visible(False)
-            else:
-                ax.set_ylabel('mean +- sd')
-                ax2.set_ylabel('sd')
+            ax2.set_xticks(np.arange(-10,10))
+            ax2.set_xticklabels(['']*20)
+        
+
+    def refresh(self):
+        sps = self.spikesorter
+        for i in range(sps.trodness):
+            self.axs[i].clear()
+            self.ax2s[i].clear()
+
+        if sps.spike_waveforms is None : 
+            self.canvas.draw()
+            return
+        
+        # recreate axes
+        
+                
         times = (np.arange(-sps.left_sweep, sps.right_sweep+1)/sps.wf_sampling_rate).rescale('ms').magnitude
         
-        # plots
         for i in range(sps.trodness):
-            ax = self.axs[i]
-            if len(sps.cluster_names) == 0:
-                slices = [ slice(None,None,None) ]
-                colors = [ 'b' ]
-            else:
-                slices = [c==sps.spike_clusters for c in sps.cluster_names if sps.active_cluster[c]]
-                colors = [ sps.cluster_colors[c] for c in sps.cluster_names if sps.active_cluster[c]]
-            
-            for sl,color in zip(slices, colors):
-                m  = np.mean(sps.spike_waveforms[sl,i,:], axis = 0)
-                sd = np.std(sps.spike_waveforms[sl,i,:], axis = 0)
-                ax.plot(times, m, linewidth=2,color = color,)
-                ax.fill_between(times, m-sd, m+sd ,
+            #~ self.axs[i].clear()
+            #~ self.ax2s[i].clear()
+            if i==0:
+                self.axs[i].set_ylabel(self.plot_parameters['estimation'])
+                self.ax2s[i].set_ylabel(self.plot_parameters['estimation'].split('+')[-1])
+        
+        # plots
+        coeff = self.plot_parameters['deviation_coeff']
+        #~ print sps.cluster_names.keys()
+        #~ print sps.median_centers.keys()
+        for c in sps.cluster_names:
+            #~ print c
+            if not sps.active_cluster[c]: continue
+            for i in range(sps.trodness):
+                if self.plot_parameters['estimation'] == 'Median + MAD':
+                    m = sps.median_centers[c][i,:]
+                    sd = sps.mad_deviation[c][i,:]
+                elif self.plot_parameters['estimation']== 'Mean + STD':
+                    m = sps.mean_centers[c][i,:]
+                    sd = sps.std_deviation[c][i,:]
+                
+                color = sps.cluster_colors[c]
+                self.axs[i].plot(times, m, linewidth=2,color = color,)
+                self.axs[i].fill_between(times, m-sd*coeff, m+sd*coeff ,
                                     color = color, alpha = .3)
-                self.axs[0].set_xlim(times[0], times[-1])
                 self.ax2s[i].plot(times, sd, linewidth=2,color = color,)
+                
+                self.axs[i].axvline(0, color = 'r', ls = '--', alpha = .7)
+                self.ax2s[i].axvline(0, color = 'r', ls = '--', alpha = .7)
+                
+        self.axs[0].set_xlim(times[0], times[-1])
+        
+        for i in range(sps.trodness):
+            self.ax2s[i].set_xticks(np.arange(-10,10))
+            self.ax2s[i].set_xticklabels(['']*20)
+
+
+        
+        #~ for i in range(sps.trodness):
+            #~ ax = self.axs[i]
+            #~ if len(sps.cluster_names) == 0:
+                #~ slices = [ slice(None,None,None) ]
+                #~ colors = [ 'b' ]
+            #~ else:
+                #~ slices = [c==sps.spike_clusters for c in sps.cluster_names if sps.active_cluster[c]]
+                #~ colors = [ sps.cluster_colors[c] for c in sps.cluster_names if sps.active_cluster[c]]
+            #~ for sl,color in zip(slices, colors):
+                #~ if self.plot_parameters['estimation'] == 'Median + MAD':
+                    #~ m  = np.median(sps.spike_waveforms[sl,i,:], axis = 0)
+                    #~ sd = np.median(np.abs(sps.spike_waveforms[sl,i,:]-m), axis = 0)/  .6745
+                #~ elif self.plot_parameters['estimation']== 'Mean + STD':
+                    #~ m  = np.mean(sps.spike_waveforms[sl,i,:], axis = 0)
+                    #~ sd = np.std(sps.spike_waveforms[sl,i,:], axis = 0)
+
+                #~ coeff = self.plot_parameters['deviation_coeff']
+                #~ ax.plot(times, m, linewidth=2,color = color,)
+                #~ ax.fill_between(times, m-sd*coeff, m+sd*coeff ,
+                                    #~ color = color, alpha = .3)
+                #~ self.axs[0].set_xlim(times[0], times[-1])
+                #~ self.ax2s[i].plot(times, sd, linewidth=2,color = color,)
 
         self.canvas.draw()
 
@@ -99,7 +157,7 @@ class AllWaveforms(SpikeSortingWidgetBase):
         self.selected_lines = None
         self.already_in_pick = False # to avoid multiple pick
         
-        self.refresh()
+        #~ self.refresh()
     
     def refresh(self):
         
@@ -108,11 +166,13 @@ class AllWaveforms(SpikeSortingWidgetBase):
         self.fig.clear()
         self.selected_lines = None
         
-        if sps.spike_waveforms is None : return
+        if sps.spike_waveforms is None : 
+            self.canvas.draw()
+            return
             
         # recreate axes
         ax = None
-        grid = GridSpec(1,sps.trodness)
+        grid = GridSpec(1,sps.trodness, wspace=0, hspace=0)
         for i in range(sps.trodness):
             ax = self.fig.add_subplot( grid[i] , sharex = ax, sharey = ax)
             self.axs.append( ax )
@@ -128,6 +188,8 @@ class AllWaveforms(SpikeSortingWidgetBase):
             ax.clear()
             ax.axvline(0, color = 'r', ls = '--', alpha = .7)
             self.lines.append([ ])
+            ax.set_xticks(np.arange(-10,10))
+            ax.set_xticklabels(['']*20)
         
         self.ploted_indices =[ ]
         for c in sps.cluster_names.keys():
@@ -153,7 +215,8 @@ class AllWaveforms(SpikeSortingWidgetBase):
     
     def refresh_selection(self):
         sps = self.spikesorter
-        if sps.spike_waveforms is None : return
+        if sps.spike_waveforms is None : 
+            return
         
         if self.selected_lines is not None:
             for i, ax in enumerate(self.axs):
