@@ -3,7 +3,7 @@ import numpy as np
 
 from sklearn.cluster import MiniBatchKMeans
 
-
+from .tools import apply_descending_sort_with_waveform
 
 
 class SklearnMiniBatchKMeans(object):
@@ -23,10 +23,11 @@ class SklearnMiniBatchKMeans(object):
                             {'name': 'init', 'type': 'list', 'value': 'k-means++', 'values':['k-means++', 'random']},
                             {'name': 'batch_size', 'type': 'int', 'value': 100},
                             {'name': 'max_iter', 'type': 'int', 'value': 300},
+                            {'name': 'descending_sort_with_waveform', 'type': 'bool', 'value': True},
                             ]
     
     def run(self, spikesorter, n_cluster =8 ,  init='k-means++', batch_size=100, 
-                            max_iter=300, ):
+                            max_iter=300, descending_sort_with_waveform = True):
         sps = spikesorter
         
         classifier = MiniBatchKMeans( k = n_cluster , init = init, 
@@ -38,6 +39,9 @@ class SklearnMiniBatchKMeans(object):
         sps.cluster_names = dict( [ (i, 'cluster #{}'.format(i))
                                     for i in np.unique(sps.spike_clusters) ] )
 
+        if descending_sort_with_waveform:
+            apply_descending_sort_with_waveform(sps)
+
 
 
 
