@@ -93,7 +93,11 @@ from methods import all_methods
 all_method_names = dict([ (method.__name__, method) for method in all_methods ])
 
 from matplotlib.cm import get_cmap
-from matplotlib.colors import ColorConverter
+#~ from matplotlib.colors import ColorConverter
+#~ import string
+from . import color_utils
+
+#~ import string
 
 from ..core import OEBase, open_db
 import neo
@@ -264,7 +268,8 @@ class SpikeSorter(object):
         self.full_band_sigs=None
         self.sig_sampling_rate = None
         self.filtered_sigs=None
-        self.spike_index_array = None
+        #~ self.spike_index_array = None
+        self.set_attr_no_check( 'spike_index_array', None)
         
         self.seg_spike_slices = { }
         self.spike_waveforms = None
@@ -288,6 +293,8 @@ class SpikeSorter(object):
         self.selected_spikes = None
         self.active_cluster = { }
         
+        self.interspike_noise_median = None
+        
         #~ self.__setattr__ = self.__setattr__after
         
         self.initialize_from_rcg(rcg)
@@ -339,6 +346,8 @@ class SpikeSorter(object):
         name = self.aliases.get(name, name)
         return object.__getattribute__(self, name)
 
+    def set_attr_no_check(self , name, val):
+        object.__setattr__(self, name, val)
 
     def check_change_on_attributes(self, name):
         """
@@ -351,19 +360,23 @@ class SpikeSorter(object):
             return 
         
         if name == 'spike_index_array':
+            object.__setattr__(self, 'spike_waveforms', None)
+            object.__setattr__(self, 'waveform_features', None)
             if self.spike_index_array is None:
                 nb_spikes = 0
             else:
                 nb_spikes = np.sum([len(pos) for pos in self.spike_index_array])
             self.spike_clusters = np.zeros(nb_spikes, dtype = int)
             self.init_seg_spike_slices()
-            object.__setattr__(self, 'spike_waveforms', None)
-            object.__setattr__(self, 'waveform_features', None)
+            object.__setattr__(self, 'interspike_noise_median', None)
         elif name == 'spike_waveforms':
             object.__setattr__(self, 'waveform_features', None)
+            object.__setattr__(self, 'interspike_noise_median', None)
+            self.recompute_cluster_center()
         elif name == 'spike_clusters':
             self.cluster_names = { }
             self.refresh_cluster_names()
+            self.recompute_cluster_center()
         #~ self.selected_spikes is None
 
     def init_seg_spike_slices(self):
@@ -565,7 +578,17 @@ class SpikeSorter(object):
                            self.wf_units =  sptr.waveforms.units
                     if 'waveform_features' in sptr.annotations:
                         features.append(sptr.annotations['waveform_features'])
-                
+                    
+                    if 'color' in unit.annotations is not None and unit.annotations['color'] is not None and\
+                            unit.annotations['color'] !='':
+                        #~ hexcolor = unit.annotations['color'].replace('#', '')
+                        #~ color = [ ]
+                        #~ for i in range(3):
+                            #~ color.append(float(string.atoi('0x'+hexcolor[i*2:i*2+2], 16))/255.)
+                        #~ self.cluster_colors[u] = color
+                        self.cluster_colors[u] = color_utils.html_to_mplRGB(unit.annotations['color'])
+                        
+                    
                 self.seg_spike_slices[s] = slice(pos, pos+nb_spike_in_segs)
                 pos += nb_spike_in_segs
                 #~ self.spike_index_array[s] = np.concatenate(spike_index)
@@ -620,6 +643,9 @@ class SpikeSorter(object):
             print step
             other.run_step(step['methodInstance'].__class__, **step['arguments'])
     
+    
+    
+    
     ## Manul clustering utilities
     def delete_one_cluster(self, c):
         # spike_index_array first
@@ -638,33 +664,7 @@ class SpikeSorter(object):
         self.init_seg_spike_slices()
 
 
-        #~ self.full_band_sigs=None
-        #~ self.sig_sampling_rate = None
-        #~ self.filtered_sigs=None
-        #~ self.spike_index_array = None
-        
-        #~ self.seg_spike_slices = { }
-        #~ self.spike_waveforms = None
-        #~ self.wf_sampling_rate = None
-        #~ self.left_sweep = None
-        #~ self.right_sweep = None
-        #~ self.waveform_features = None
-        #~ self.feature_names = None
-        
-        #~ self.spike_clusters = None
-        #~ self.cluster_names = { }
-        #~ self.spike_clusters_probabilistic = None
-        
-        #~ self.seg_t_start = None
-        #~ self.seg_t_stop = None
-        #~ self.wf_units = None
-        
-        #~ self.cluster_colors = { }
-        #~ self.displayed_subset_size = 100
-        #~ self.cluster_displayed_subset = { }
-        #~ self.selected_spikes = None
-
-    
+   
     def add_one_spike(self, s, time, c = 0):
         if self.filtered_sigs is None:
             raise Exception('No signals')
@@ -683,20 +683,24 @@ class SpikeSorter(object):
         self.spike_index_array[s] = np.concatenate( (self.spike_index_array[s], [index]))
         
         # tricky insertion at the good place
-        all_wf = [ ]
-        all_sc = [ ]
-        for i in range(s):
-            all_wf.append(self.spike_waveforms[self.seg_spike_slices[i]])
-            all_sc.append(self.spike_clusters[self.seg_spike_slices[i]])
-        all_wf.append(self.spike_waveforms[self.seg_spike_slices[s]])
-        all_wf.append(wf)
-        all_sc.append(self.spike_clusters[self.seg_spike_slices[s]])
-        all_sc.append([c])
-        for i in range(s+1, len(self.segs)):
-            all_wf.append(self.spike_waveforms[self.seg_spike_slices[i]])
-            all_sc.append(self.spike_clusters[self.seg_spike_slices[i]])
-        self.spike_waveforms = np.concatenate(all_wf)
-        self.spike_clusters = np.concatenate(all_sc)
+        if self.spike_waveforms is not None:
+            all_wf = [ ]
+            for i in range(s):
+                all_wf.append(self.spike_waveforms[self.seg_spike_slices[i]])
+            all_wf.append(self.spike_waveforms[self.seg_spike_slices[s]])
+            all_wf.append(wf)
+            self.spike_waveforms = np.concatenate(all_wf)
+        
+        if self.spike_clusters is not None:
+            all_sc = [ ]
+            for i in range(s):
+                all_sc.append(self.spike_clusters[self.seg_spike_slices[i]])
+            all_sc.append(self.spike_clusters[self.seg_spike_slices[s]])
+            all_sc.append([c])
+            for i in range(s+1, len(self.segs)):
+                all_sc.append(self.spike_clusters[self.seg_spike_slices[i]])
+            self.spike_clusters = np.concatenate(all_sc)
+            
         self.init_seg_spike_slices()
     
 
@@ -714,13 +718,75 @@ class SpikeSorter(object):
         for c in to_pop:
             self.cluster_names.pop(c)
         self.cluster_names[n] = u'regrouped small cluster'
-        self.refresh_colors()
+        self.refresh_colors(reset = False)
+    
+    
+    #####
+    ##  Stat utiliities
+    def recompute_interspike_noise(self, n  = 1000, maxiter = 5):
+        """
+        this take small chunk of signal btween spike same size as waveform to estimate the noise.
+        """
+        
+        self.interspike_noise= np.empty( (len(self.rcs), len(self.segs)), dtype = object)
+        
+        sr = self.sig_sampling_rate
+        swl = self.left_sweep
+        swr = self.right_sweep
+        wsize = swl + swr + 1
+        trodness = len(self.rcs)
+        
+        def isnear(a,b, dist):
+            return np.array([ np.any(abs(b-e)<dist) for e in a ], dtype = bool)
+        
+        noise_chunk  = np.empty((n, trodness, wsize), dtype = float)
+        ind = 0
+        for s, seg in enumerate(self.segs):
+            n_by_seg = max(n//len(self.segs),2)
+            
+            rand_size = self.full_band_sigs[0,s].size-wsize-1
+            #take random pos
+            pos = np.random.randint(rand_size, size = n_by_seg)+swl
+            for iter in range(maxiter):
+                # check is is not overlapping with spikes
+                #~ print iter
+                near = isnear(pos, self.spike_index_array[s], wsize*2)
+                if np.sum(near)==0: break
+                pos[near] = np.random.randint(rand_size, size =  np.sum(near))+swl
+            pos = pos[~near]
+            
+            for p in pos:
+                for i in range(len(self.rcs)):
+                    noise_chunk[ind, i, :] = self.filtered_sigs[i,s][p-swl: p+swr+1]
+                ind += 1
+        
+        noise_chunk = noise_chunk[:ind]
+        
+        self.interspike_noise_median = np.median(noise_chunk, axis = 0)
+        self.interspike_noise_mad = np.median(np.abs(noise_chunk-self.interspike_noise_median), axis = 0)/  .6745
+        
+    
+    def recompute_cluster_center(self):
+        self.median_centers = { }
+        self.mean_centers = { }
+        self.mad_deviation = { }
+        self.std_deviation = { }
+        
+        if self.spike_waveforms is None: return
+        print 'recompute_cluster_center'
+        for c in self.cluster_names:
+            ind = c==self.spike_clusters
+            self.median_centers[c]= np.median(self.spike_waveforms[ind,:,:], axis=0)
+            self.mean_centers[c]= np.mean(self.spike_waveforms[ind,:,:], axis=0)
+            self.mad_deviation[c]= np.median(np.abs(self.spike_waveforms[ind,:,:]-self.median_centers[c]), axis = 0)/  .6745
+            self.std_deviation[c]= np.std(self.spike_waveforms[ind,:,:], axis=0)
+
     
     ####
     ## Plot utilities
     def check_display_attributes(self):
-        self.refresh_cluster_names()
-        self.refresh_colors()
+        self.refresh_cluster_names(reset = False)
+        self.refresh_colors(reset = False)
         #self.refresh_displayed_subset(self.displayed_subset_size)
         if self.selected_spikes is None and self.nb_spikes is not None:
             self.selected_spikes = np.zeros( (self.nb_spikes), dtype = bool)
@@ -734,14 +800,16 @@ class SpikeSorter(object):
                 self.random_display_subset(c)
         
 
-    def refresh_cluster_names(self):
+    def refresh_cluster_names(self,  reset = False):
         if self.spike_clusters is None:
             self.cluster_names = { }
             return
         clusters = np.unique(self.spike_clusters)
         for c in np.setdiff1d(self.cluster_names.keys(), clusters):
             self.cluster_names.pop(c)
-            
+        
+        if reset:
+            self.cluster_names = { }
         
         for c in clusters:
             if c not in self.cluster_names:
@@ -750,14 +818,27 @@ class SpikeSorter(object):
                 else:
                     self.cluster_names[-1] = 'trash'
 
-    def refresh_colors(self):
-        self.refresh_cluster_names()
-        self.cluster_colors = { }
-        if self.spike_clusters is not None:
-            cmap = get_cmap('jet' , len(self.cluster_names)+3)
-            for i , c in enumerate(self.cluster_names.keys()):
-                self.cluster_colors[c] = ColorConverter().to_rgb( cmap(i+2) )
-        self.cluster_colors[-1] = ColorConverter().to_rgb( 'k' ) #trash
+    def refresh_colors(self, reset = True):
+        self.refresh_cluster_names(reset = reset)# TODO : why this
+        if self.spike_clusters is  None:
+            self.cluster_colors = { }
+            return
+        
+        clusters = self.cluster_names.keys()
+        for c in np.setdiff1d(self.cluster_colors.keys(), clusters):
+            self.cluster_colors.pop(c)
+        
+        if reset:
+            self.cluster_colors = { }
+
+        cmap = get_cmap('jet' , len(self.cluster_names)+3)
+        for i , c in enumerate(self.cluster_names.keys()):
+            if c in self.cluster_colors: continue
+            #~ self.cluster_colors[c] = ColorConverter().to_rgb( cmap(i+2) )
+            self.cluster_colors[c] = color_utils.mpl_to_mplRGB( cmap(i+2) )
+        #~ self.cluster_colors[-1] = ColorConverter().to_rgb( 'k' ) #trash
+        self.cluster_colors[-1] = color_utils.mpl_to_mplRGB( 'k' ) #trash
+        
     
     def refresh_displayed_subset(self, displayed_subset_size = 100):
         self.displayed_subset_size = displayed_subset_size
@@ -771,7 +852,9 @@ class SpikeSorter(object):
     
     def on_clusters_activation_changed(self):
         for c, activ in self.active_cluster.items():
-            if activ and self.cluster_displayed_subset[c].size ==0:
+            if activ and c not in self.cluster_displayed_subset:
+                self.random_display_subset(c)
+            elif activ and self.cluster_displayed_subset[c].size ==0:
                 self.random_display_subset(c)
             elif not activ and self.cluster_displayed_subset[c].size >0:
                 self.cluster_displayed_subset[c]  = np.array([ ], dtype = int)
@@ -806,12 +889,22 @@ class SpikeSorter(object):
         rcg.units = [ ]
         
         self.refresh_cluster_names()
+        self.refresh_colors()
         for c, name in self.cluster_names.items():
-            unit = neo.Unit(name = name)
+            #~ color = '#'
+            #~ for e in self.cluster_colors[c]:
+                #~ colorhex = str(hex(int(e*255))).replace('0x','')
+                #~ if len(colorhex) == 1:
+                    #~ colorhex = '0'+colorhex
+                #~ color += colorhex
+            color = color_utils.mpl_to_html(self.cluster_colors[c])
+
+            unit = neo.Unit(name = name, color = color)
             rcg.units.append(unit)
             for s, seg in enumerate(bl.segments):
                 sptr = neo.SpikeTrain(self.get_spike_times(s, c, units = 's'),
-                                    t_start = self.seg_t_start[s], t_stop = self.seg_t_stop[s])
+                                    t_start = self.seg_t_start[s], t_stop = self.seg_t_stop[s],
+                                    color = color)
                 if with_waveforms:
                     sptr.waveforms = self.get_spike_waveforms(s, c) * self.wf_units
                     sptr.sampling_rate = self.wf_sampling_rate
